// (c) Dominic Beesley 2020
// 
// SC/MP NIBL to USB Serial adaptor
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.
//
// Requires:
//  Arduino DUE (or other multi-serial-port Arduino)
//
//  WARNING the DUE is a 3V device a dropper resistors or level converters are recommended
//  no liability will be assumed!
//
//  SC/MP nibble connections:
//  Arduino Pin   SC/MP Pin   Notes
//  ======================================================
//  19 (RX1)      19 (F0)     This via an inveter
//  18 (TX1)      18 (SB)
//  2             21 (F1)     "relay"
//  9             19          Direct connection to SC/MP TX
//  8 --> 19      -           Loop back to RX1

#define PIN_LP_IN 9
#define PIN_LP_OUT 8
#define PIN_RELAY 2

void setup() {

  pinMode(PIN_LP_IN, INPUT);
  pinMode(PIN_LP_OUT, OUTPUT);
  pinMode(PIN_RELAY, INPUT);
  
  Serial.begin(9600);
  Serial1.begin(220, SERIAL_8N1);
}

bool breaking = 0;

void loop() {
  
  while (Serial1.available()>0) 
    Serial.write(Serial1.read() & 0x7F);

  if (breaking) {
    if (digitalRead(PIN_RELAY))
    {
      breaking = 0;
    } else {
      Serial1.write((const uint8_t)0);
      delay(100);
    }
  } else if (Serial.available()>0) {
    if (digitalRead(PIN_RELAY)) {
      Serial1.write(Serial.read());
      delay(100);
    } else if (Serial.peek() == 3) {
      //break
      //keep writing 0 until the RELAY line pops back up
      Serial.read(); // eat ^C
      breaking = 1;
    }
  }

  digitalWrite(PIN_LP_OUT, !digitalRead(PIN_LP_IN));
}
